'use strict';

const core$1 = require('@tiptap/core');
const CodeBlock = require('@tiptap/extension-code-block');
const state = require('@tiptap/pm/state');
const view = require('@tiptap/pm/view');
const core = require('shiki/core');
const css = require('shiki/langs/css.mjs');
const html = require('shiki/langs/html.mjs');
const javascript = require('shiki/langs/javascript.mjs');
const json = require('shiki/langs/json.mjs');
const markdown = require('shiki/langs/markdown.mjs');
const python = require('shiki/langs/python.mjs');
const typescript = require('shiki/langs/typescript.mjs');
const vue = require('shiki/langs/vue.mjs');
const vitesseDark = require('shiki/themes/vitesse-dark.mjs');
const vitesseLight = require('shiki/themes/vitesse-light.mjs');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const CodeBlock__default = /*#__PURE__*/_interopDefaultCompat(CodeBlock);
const css__default = /*#__PURE__*/_interopDefaultCompat(css);
const html__default = /*#__PURE__*/_interopDefaultCompat(html);
const javascript__default = /*#__PURE__*/_interopDefaultCompat(javascript);
const json__default = /*#__PURE__*/_interopDefaultCompat(json);
const markdown__default = /*#__PURE__*/_interopDefaultCompat(markdown);
const python__default = /*#__PURE__*/_interopDefaultCompat(python);
const typescript__default = /*#__PURE__*/_interopDefaultCompat(typescript);
const vue__default = /*#__PURE__*/_interopDefaultCompat(vue);
const vitesseDark__default = /*#__PURE__*/_interopDefaultCompat(vitesseDark);
const vitesseLight__default = /*#__PURE__*/_interopDefaultCompat(vitesseLight);

const style = `
.tiptap .shiki {
  border-radius: 4px;
  padding: 8px;
}
.tiptap .shiki:focus-visible {
  outline: 2px solid transparent;
  outline-offset: 2px;
}

html.dark .tiptap .shiki,
html.dark .tiptap .shiki span {
  color: var(--shiki-dark) !important;
  background-color: var(--shiki-dark-bg) !important;
  font-style: var(--shiki-dark-font-style) !important;
  font-weight: var(--shiki-dark-font-weight) !important;
  text-decoration: var(--shiki-dark-text-decoration) !important;
}
`;

const codeBlock = CodeBlock__default.extend({
  addOptions() {
    return {
      ...this.parent?.(),
      languages: [css__default, html__default, javascript__default, json__default, markdown__default, python__default, typescript__default, vue__default],
      themes: {
        light: vitesseLight__default,
        dark: vitesseDark__default
      }
    };
  },
  addStorage() {
    const highlighter = core.createHighlighterCoreSync({
      langs: this.options.languages,
      themes: Object.values(this.options.themes),
      engine: core.createJavaScriptRegexEngine()
    });
    const languages = Array.from(new Set(this.options.languages.map((language) => language.map((item) => ({
      name: item.name,
      displayName: item.displayName
    })).filter((item) => !!item.displayName)).flat(2))).reduce((languages2, item) => {
      if (languages2.find((language) => language.name === item.name)) {
        return languages2;
      }
      languages2.push(item);
      return languages2;
    }, []);
    return {
      highlighter,
      languages
    };
  },
  addProseMirrorPlugins() {
    const getDecorations = (doc) => {
      const decorations = [];
      core$1.findChildren(doc, (node) => node.type.name === this.name).forEach(({ node, pos }) => {
        const language = node.attrs.language ?? "text";
        const hast = this.storage.highlighter.codeToHast(node.textContent, {
          lang: language,
          themes: this.options.themes
        });
        const preNode = hast.children[0];
        decorations.push(view.Decoration.node(pos, pos + node.nodeSize, preNode.properties));
        let from = pos + 1;
        const lines = preNode.children[0].children;
        for (const line of lines) {
          if (line.children?.length) {
            let lineFrom = from;
            line.children?.forEach((node2) => {
              const nodeLen = node2.children[0].value.length;
              decorations.push(view.Decoration.inline(lineFrom, lineFrom + nodeLen, node2.properties));
              lineFrom += nodeLen;
            });
            from = lineFrom;
          } else if (line.type === "text") {
            from += line.value.length;
          }
        }
      });
      return view.DecorationSet.create(doc, decorations);
    };
    return [
      new state.Plugin({
        key: new state.PluginKey(this.name),
        state: {
          init(_, { doc }) {
            return getDecorations(doc);
          },
          apply: (tr, value, _, { doc: newDoc }) => {
            if (!tr.docChanged) {
              return value.map(tr.mapping, tr.doc);
            }
            const changedRanges = core$1.getChangedRanges(tr);
            let hasChangedCodeBlock = false;
            for (const range of changedRanges) {
              hasChangedCodeBlock = !!core$1.findChildrenInRange(newDoc, range.newRange, (node) => node.type.name === this.name).length;
              if (hasChangedCodeBlock) {
                return getDecorations(newDoc);
              }
            }
            return value.map(tr.mapping, tr.doc);
          }
        },
        props: {
          decorations(state) {
            return this.getState(state);
          }
        }
      })
    ];
  },
  onBeforeCreate() {
    core$1.createStyleTag(style, void 0, this.name);
  }
});

exports.codeBlock = codeBlock;
